// ScolPlugin.cpp : Implementation of CScolPlugin

#include "stdafx.h"
#include "NewATLScol.h"
#include "ScolPlugin.h"


/////////////////////////////////////////////////////////////////////////////
// CScolPlugin

CScolPlugin::CScolPlugin(void)
{
	// Request a Window Handler :
	// as control may be windowless, we must say that we need a window
	m_bWindowOnly = TRUE;

	m_bInPlaceActive=FALSE;
	m_bUIActive=FALSE;

	// Init Member vars
	m_boolForceInstall=FALSE;
	m_bstrDownloadURL="";
	m_bstrScolVersionNeeded="";
	m_longDownloadSize=0;

	// Store Singleton
	MyScolPlugSingleton=this;
	// Create new Scol Access Object
	MyScolInterface = new ScolInterface();
	// Set JS CallBack Function
	MyScolInterface->set_SendMessageCallBack((void*)&MyCallBackForScol);
	MyScolInterface->set_ScolEndCallBack((void*)&MyEndCallBackForScol);
	
	// Increment Global Instance counter
	NBPlugInstance++;

	// display Warning if there is more than One Instance
	// this limitation is because of Scol, not Mozilla API /IE COM API
	// - Plugin Hwnd and CallBack are shared resources in Scol DLL !!!!
//	if (NBPlugInstance>1)
//		::MessageBox(NULL,"You can only launch one instance of Scol","Sorry !",MB_OK|MB_ICONEXCLAMATION);

 
}

CScolPlugin::~CScolPlugin(void)
{
	// Decrement Global Counter
	if (NBPlugInstance>0)
		NBPlugInstance--;
	
	//::MessageBox(NULL,"CScol Destructor","DEBUG",0);
	// release MyScolInterface
	delete MyScolInterface;
	MyScolPlugSingleton=NULL;
}

STDMETHODIMP CScolPlugin::LaunchMachine(BSTR *CmdLine, long CmdShow, long MemSize, long *result)
{
	USES_CONVERSION;
	char * cmdl;

	cmdl=(char *)OLE2A(*CmdLine);

	// Modify Download URL
	if (m_bstrDownloadURL.Length()>0)
		MyScolInterface->set_DownloadURL(OLE2A(m_bstrDownloadURL));

	// Modify Download size
	if (m_longDownloadSize>0)
		MyScolInterface->set_DownloadSize(m_longDownloadSize);

	// Modify Version Needed
	if (m_bstrScolVersionNeeded.Length()>0)
		MyScolInterface->set_ScolVersionNeeded(OLE2A(m_bstrScolVersionNeeded));

	// Force Install if Needed
	if (m_boolForceInstall)
		MyScolInterface->ForceInstall();

	*result=MyScolInterface->LaunchMachine(cmdl,CmdShow,MemSize,m_hWndCD);
	
	InPlaceActivate(OLEIVERB_UIACTIVATE);		

	return S_OK;
}


STDMETHODIMP CScolPlugin::SendMessage(BSTR *msg, long *result)
{
	char * MsgToScol;
	USES_CONVERSION;
	MsgToScol=(char *)OLE2A(*msg);

	if (MyScolInterface)
		*result=MyScolInterface->SendMessageToScol(MsgToScol);
	return S_OK;
}


///////////////////////////////////////////////////////////////////////
// Property Accessors for ScolVersionNeeded Property
STDMETHODIMP CScolPlugin::get_ScolVersionNeeded(BSTR *pVal)
{
	*pVal = m_bstrScolVersionNeeded.Copy();
	return S_OK;
}

STDMETHODIMP CScolPlugin::put_ScolVersionNeeded(BSTR newVal)
{
	m_bstrScolVersionNeeded=newVal;
	return S_OK;
}

///////////////////////////////////////////////////////////////////////
// Property Accessors for DownloadURL Property
STDMETHODIMP CScolPlugin::get_DownloadURL(BSTR *pVal)
{
	*pVal = m_bstrDownloadURL.Copy();
	return S_OK;
}

STDMETHODIMP CScolPlugin::put_DownloadURL(BSTR newVal)
{
	USES_CONVERSION;
	char * NewDownloadURL;

	
	
	m_bstrDownloadURL=newVal;
	if (MyScolInterface)
	{
		NewDownloadURL=OLE2A(m_bstrDownloadURL);
		MyScolInterface->set_DownloadURL(NewDownloadURL);
	};
	return S_OK;
}

/**********************************************************************************/
/* This section is here to provide a mapping C Function -> Object member function */

/**********************************************************************************/


int MyCallBackForScol(char* Msg)
{
return MyScolPlugSingleton->SendMessageToJS(Msg);
}

int MyEndCallBackForScol(int Reason)
{
ScolQuit=TRUE;
return MyScolPlugSingleton->SendEndEventToJS(Reason);
}


int CScolPlugin::SendEndEventToJS(int Reason)
{
	Fire_EndMachine();
	//::MessageBox(NULL,"Scol End Message Received","DEBUG",NULL);
	return 0;
}

int CScolPlugin::SendMessageToJS(char *JSMessage)
{
  USES_CONVERSION;
  BSTR bstrJSMessage;
  bstrJSMessage = A2BSTR(JSMessage);
  Fire_Message(&bstrJSMessage);
  //::MessageBox(NULL,JSMessage,"VC:Message from Scol to JS",NULL);
  return 0;
}


STDMETHODIMP CScolPlugin::get_ForceInstall(BOOL *pVal)
{
	// TODO: Add your implementation code here
	*pVal=(BOOL)m_boolForceInstall;
	return S_OK;
}

STDMETHODIMP CScolPlugin::put_ForceInstall(BOOL newVal)
{
	// TODO: Add your implementation code here
	m_boolForceInstall=(BOOL)newVal;
	return S_OK;
}



STDMETHODIMP CScolPlugin::get_DownloadSize(long *pVal)
{
	// TODO: Add your implementation code here

	*pVal = m_longDownloadSize;
	return S_OK;
}

STDMETHODIMP CScolPlugin::put_DownloadSize(long newVal)
{
	// TODO: Add your implementation code here

	m_longDownloadSize = newVal;
	if (MyScolInterface)
	{
		MyScolInterface->set_DownloadSize(newVal);
	};
	return S_OK;
}
