// ScolInterface.cpp: implementation of the ScolInterface class.
//
//////////////////////////////////////////////////////////////////////

#include "ScolInterface.h"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////
// Default Download URL
char DefDownloadURL[]="http://www.cryo-networks.com/files/scp10.exe";

//////////////////////////////////////////////////////////////////////
// Default Download Size
long DefDownloadSize=1200000;

//////////////////////////////////////////////////////////////////////
// handle on Scol Dll
HINSTANCE hScolDLL;


//////////////////////////////////////////////////////////////////////
// Definition des interfaces avec la VM Scol
int (*SCOLWinMainActiveX)(HANDLE this_inst, HANDLE prev_inst, LPSTR cmdline,int cmdshow, void *,int size,HWND axhwnd,void *);
int (*EndScolMachine)(int n);
int (*ScolSendMessage)(char * buf);
//void (*CBEndScol)(void (*cb)(int reason));
void (*CBEndScol)(void *);

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ScolInterface::ScolInterface()
{
	//////////////////////////////
	// init 
	//
	m_boolForceInstall=false;
	m_boolFakeInstance=FALSE;

	// Default Download URL
	strcpy(m_strDownloadURL,DefDownloadURL);

	// Default Download Size
	m_longDownloadSize = DefDownloadSize;

	/*SCOLWinMainActiveX=NULL;
    EndScolMachine=NULL;
    ScolSendMessage=NULL;
    CBEndScol=NULL;*/

	// Initialise Scol Library :
	// Load Scol DLL if not already done
	if (hScolDLL==NULL)
		m_boolScolLoadSuccess=LoadScolLib();
	else
		m_boolScolLoadSuccess=true;

}

ScolInterface::~ScolInterface()
{
	// Shut Down Scol if not already Done
	if ((EndScolMachine)&&(ScolQuit==FALSE))
		EndScolMachine(0);

	// init LoadScol flag
	m_boolScolLoadSuccess=false;
	
	if (m_boolFakeInstance)
		NBPlugFakeInstance--;

	// If no more instance then Unload the DLL
	if (NBPlugInstance<=0)
	{

		// Send EndScol Event to JavaScript if not allready Done
		if ((CBEndScol)&&(ScolQuit==FALSE))
			CBEndScol(NULL);

		// Release Fonctions Pointers
		SCOLWinMainActiveX=NULL;
		EndScolMachine=NULL;
		ScolSendMessage=NULL;
		CBEndScol=NULL;
		
		//Clear Message Stack
		SkipMessages();

		// Unload Library
		// Don't Unload Library to avoid Scol to Crash
		//FreeLibrary(hScolDLL);
		hScolDLL=NULL;
	}
}

void ScolInterface::ForceInstall()
{
	m_boolForceInstall=true;
}
/////////////////////////////////////////////////////////////////////
// Write Accessor for m_strDownloadURL member var
void ScolInterface::set_DownloadURL(char *DWURL)
{
	sprintf(m_strDownloadURL,"%s",DWURL);
}

/////////////////////////////////////////////////////////////////////
// Write Accessor for m_longDownloadSize member var
void ScolInterface::set_DownloadSize(long Size)
{
	m_longDownloadSize = Size;
}

/////////////////////////////////////////////////////////////////////
// Write Accessor for m_strScolVersionNeeded member var
void ScolInterface::set_ScolVersionNeeded(char *SVersion)
{
	sprintf(m_strScolVersionNeeded,"%s",SVersion);
}

/////////////////////////////////////////////////////////////////////
// Read Accessor for m_strDownloadURL member var
char* ScolInterface::get_DownloadURL()
{
	return m_strDownloadURL;
}

/////////////////////////////////////////////////////////////////////
// Read Accessor for m_longDownloadSize member var
long ScolInterface::get_DownloadSize()
{
	return m_longDownloadSize;
}

/////////////////////////////////////////////////////////////////////
// Read Accessor for m_strScolVersionNeeded member var
char* ScolInterface::get_ScolVersionNeeded()
{
	return m_strScolVersionNeeded;
}

////////////////////////////////////////////////////////////////////
// Search for Scol Dll, Load it, and get Entry points
// Return true if everything is OK
// Return false if Dll is not found or LoadLibrary failed
bool ScolInterface::LoadScolLib()
{
  char MyPath[1024],*path;
  char name[1024];

  long msize=1024;
  int i;


  ///////////////////////////////////////
  // Look for Scol Dll Path
  // New Detection method
  if (RegQueryValue(HKEY_CLASSES_ROOT,"scol_file\\shell\\open\\command",MyPath,&msize)!=ERROR_SUCCESS)
  {
    if (RegQueryValue(HKEY_CLASSES_ROOT,"scm_auto_file\\shell\\open\\command",MyPath,&msize)!=ERROR_SUCCESS)
	  {
		SCOLWinMainActiveX=NULL;
		EndScolMachine=NULL;
		ScolSendMessage=NULL;
		CBEndScol=NULL;
		return false;
	  }
  }

  // recherche de usmwin.exe
  //$BLG
  //if (!strstr(MyPath,"usmwin.exe")) 
  if ((!strstr(MyPath,"usmwin.exe")) && (!strstr(MyPath,"scol.exe")))
	{
	SCOLWinMainActiveX=NULL;
	EndScolMachine=NULL;
	ScolSendMessage=NULL;
	CBEndScol=NULL;
	return false;
	}
  
  i=msize;
  while (i && (MyPath[i]!='\\') && (MyPath[i]!='/')) i--;

  if (MyPath[i]=='\\' || MyPath[i]=='/') 
  {
    MyPath[i+1]=0;
    if (MyPath[0]=='"') 
      path=MyPath+1;
    else 
      path=MyPath;
  }

  ////////////////////////////////////////
  // Search for Dll
  if (!getScolDllname (path,name)) 
  {
    ::MessageBox(NULL,"Couldn't find the scol dll. (may be a missing usm.ini)","Error...",0);
    return false;
  }

  strcat(path,name);


  ///////////////////////////////////////
  // Load the Dll in memory and get entry points
  hScolDLL=LoadLibrary(path);
  if (!hScolDLL) 
  {
    ::MessageBox(NULL,"Couldn't load the scol dll.",path,0);
    SCOLWinMainActiveX=NULL;
    EndScolMachine=NULL;
    ScolSendMessage=NULL;
    CBEndScol=NULL;
    return false;
  }
  else
  {
	// Get entry points 
	// Main function : Lauch and defines Message CB (Scol->Plugin->JS)
    SCOLWinMainActiveX=(int (__cdecl *)(void *,void *,char *,int,void *,int,struct HWND__ *,void *))GetProcAddress(hScolDLL,"SCOLWinMainActiveX");
	if (!SCOLWinMainActiveX)
		{
		::MessageBox(NULL,"ScolWinMainActiveX Entry Point not found!","Error",0);
		return false;
		};
	// Function to Shut-Down Scol
    EndScolMachine=(int (__cdecl *)(int))GetProcAddress(hScolDLL,"EndScolMachine");
	if (!EndScolMachine)
		{
		::MessageBox(NULL,"EndScolMachine Entry Point not found!","Error",0);
		return false;
		};
	// Function to Send Message To Scol
    ScolSendMessage=(int(__cdecl *)(char *))GetProcAddress(hScolDLL,"ScolSendMessage");
	if (!ScolSendMessage)
		{
		::MessageBox(NULL,"ScolSendMessage Entry Point not found!","Error",0);
		return false;
		};
	// CB used by Scol to notify he is closing
	CBEndScol=(void (__cdecl *)(void *))GetProcAddress(hScolDLL,"CBEndScol");
	if (!CBEndScol)
		{
		::MessageBox(NULL,"CBEndScol Entry Point not found!","Error",0);
		return false;
		};
  }
  return true;
}

/////////////////////////////////////////////////////////////////////
// Search for Scol Dll in the specified path
// Return true if everything is OK
// Return false if Dll is not found 
bool ScolInterface::getScolDllname (char* path,char* name)
{
  char buf[1024];
  FILE* ini;
  bool cont=false;
  char line[1024],*p;

  strcpy (buf,path);
  strcat (buf,"usm.ini");
  ini=fopen(buf,"rt");
  if (!ini) return false;
  
  while (!feof(ini))
  {
    fgets(line,1024,ini);
    p=line;
    while (*p==32 || *p=='\t') p++;
    if (!strnicmp(p,"scol ",5))
    {
      fclose(ini);
      p+=5;
      while (*p==32) p++;
      if (!*p)
        return false;
      strcpy(name,p);
      p=name;
      p+=strlen(name);
      while (*p!=32 && p>name) p--;
      if (*(p)==32) *(p)=0;
      else return false;
      return true;
    }
  }
  fclose(ini);
  return false;
}



//////////////////////////////////////////////////////////////////////////////////
// Set CallBack Ptr for Messages Scol -> JavaScript
// Called from NS Plugin implementation Class
void ScolInterface::set_SendMessageCallBack(void * CB)
{
	m_ptrSendMsgCallBack=CB;
}


//////////////////////////////////////////////////////////////////////////////////
// Set CallBack Ptr for Scol End -> JavaScript
// Called from NS Plugin implementation Class
void ScolInterface::set_ScolEndCallBack(void * CB)
{
	m_ptrEndScolCallBack=CB;
}


//////////////////////////////////////////////////////////////////////////////////
// Launch Scol Machine :
//
long ScolInterface::LaunchMachine(const char *CmdLine, long CmdShow, long MemSize, HWND PlugWin)
{
//  char * cmdl;
  int res;
//  char buf[1024];
  char Cmdl[1024];

  m_hwnd=PlugWin;
  //////////////////////////////////////////////
  // verify Scol Installation
  // If Scol is Not Loaded then lauch Installer
  if ((!m_boolScolLoadSuccess)||(m_boolForceInstall))
  {
	ScolInstaller* MyScolInstaller;

	// Create new Scol Installer
	MyScolInstaller = new ScolInstaller(m_strDownloadURL, m_longDownloadSize);
	
	// Launch Installation
	if (MyScolInstaller->LaunchInstaller(PlugWin))
		{
		delete MyScolInstaller;
		// Initialise Scol Library
		m_boolScolLoadSuccess=LoadScolLib();
		if (!m_boolScolLoadSuccess)
			return !S_OK;
		}
	else
		{
		delete MyScolInstaller;
		::MessageBox(PlugWin,"Installation of Scol failed!","ERROR",MB_ICONERROR);
		return !S_OK;
		};
  }

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // Dirty Patch : 
  // decrement Instance count if Scol Plugin
  // was only called for testing installation
  //-----------------------------------------------------------------------------------------------
  // Reason : in IE6 Detructor of first instance is called after constructor of second instance ...
  //-----------------------------------------------------------------------------------------------
  // Tests if the string _load is in the command line :
  // - if Yes => This is a real new instance
  // - if No => Plugin was only called to test version
  if ( (!strstr(CmdLine,"%5fload+%22")) && (!strstr(CmdLine,"_load")) )
  {
		m_boolFakeInstance=TRUE;
		NBPlugFakeInstance++;
		//char DbgMessage[255];
		//sprintf(DbgMessage, "Fake LaunchMachine : nb Total=%d, nb Fake=%d",NBPlugInstance,NBPlugFakeInstance);
		//::MessageBox(PlugWin,DbgMessage,"DEBUG",MB_ICONERROR);
  }

  /////////////////////////////////////////////
  // verify Number of Instance of the Plugin
  // Only One Instance of the Plugin can be Created by Process
  // The number tested is : Total Number of Instances - number of instance only created for testing version
  if ((NBPlugInstance-NBPlugFakeInstance)>1) 
	{
	  ::MessageBox(PlugWin,"Only one Instance of the Scol PlugIn can be loaded","ERROR",MB_ICONERROR);
	  return -1;
	} 

  
  SkipMessages();

  ////////////////////////////////////////////
  // LaunchMachine
  strcpy(Cmdl,CmdLine);
  if (SCOLWinMainActiveX)	  
	res=SCOLWinMainActiveX(PlugInDllHINSTANCE,NULL,Cmdl,CmdShow,NULL,MemSize,PlugWin,m_ptrSendMsgCallBack);
	//res=SCOLWinMainActiveX(NULL,NULL,Cmdl,CmdShow,NULL,MemSize,PlugWin,m_ptrSendMsgCallBack);
  else
	{
	m_boolScolStartSuccess=false;
    return -1;
	}

  if (res) 
  { // Set EndScol CallBack
	ScolQuit=FALSE;
	m_boolScolStartSuccess=true;
	if (CBEndScol)
		CBEndScol(m_ptrEndScolCallBack);
  }
  else 
  {
	m_boolScolStartSuccess=false;
    return -1;
  }

  return res;
}

///////////////////////////////////////////////////////////////////////////////
// Send Message to Scol 
long ScolInterface::SendMessageToScol(const char *Msg)
{
	// verify that Scol is Running
	if (!m_boolScolStartSuccess)
		return !S_OK;

	// If OK, send message thought Dll export function
	char ScolMsg[1024];
	strcpy(ScolMsg,Msg);
	if (ScolSendMessage)
		return (ScolSendMessage(ScolMsg));
	else
		return -1;
}

///////////////////////////////////////////////////////////////////////////////
// 
void ScolInterface::SkipMessages()
{
  MSG msg;
  BOOL bRet;
  int i;

  i=0;

  while( (bRet = PeekMessage(&msg,NULL,0,0,PM_REMOVE))!=0)
	//while( (bRet = GetMessage(&msg, m_hwnd, 0, 0 )) != 0)
	{ 
		if (bRet == -1)
		{
			// handle the error and possibly exit
		}
		else
		{
			DispatchMessage(&msg); 
		}
	}


/* Old implementation */
/*  for (int i=0;i<50;i++)
  {
    if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }
  }*/


}